@@ -174,6 +174,9 @@ sub local_shard_data {
 
     my %results;
     foreach my $shard ( @{ $shardres } ) {
+        # Skip unallocated shards
+        next unless $shard->{id};
+
         # Skip unless this shard is allocated to this shard
         next unless $shard->{id} eq $id;
 
@@ -288,7 +291,7 @@ es-graphite-dynamic.pl - Dynamically gather metrics and send to graphite
 
 =head1 VERSION
 
-version 7.8
+version 7.9
 
 =head1 SYNOPSIS
 
@@ -422,7 +425,7 @@ Brad Lhotsky <brad@divisionbyzero.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2020 by Brad Lhotsky.
+This software is Copyright (c) 2021 by Brad Lhotsky.
 
 This is free software, licensed under:
 
